import { isLeafType, parse, print, TypeInfo, visit, Kind, isNamedType, } from 'graphql';
import { getFieldDef } from 'graphql/execution/execute';
export function fillLeafs(schema, docString, getDefaultFieldNames) {
    const insertions = [];
    if (!schema || !docString) {
        return { insertions, result: docString };
    }
    let ast;
    try {
        ast = parse(docString);
    }
    catch (error) {
        return { insertions, result: docString };
    }
    const fieldNameFn = getDefaultFieldNames || defaultGetDefaultFieldNames;
    const typeInfo = new TypeInfo(schema);
    visit(ast, {
        leave(node) {
            typeInfo.leave(node);
        },
        enter(node) {
            typeInfo.enter(node);
            if (node.kind === 'Field' && !node.selectionSet) {
                const fieldType = typeInfo.getType();
                const selectionSet = buildSelectionSet(getFieldDef(schema, field.type, node), isFieldType(fieldType), fieldNameFn, schema);
                if (selectionSet && node.loc) {
                    const indent = getIndentation(docString, node.loc.start);
                    insertions.push({
                        index: node.loc.end,
                        string: ' ' + print(selectionSet).replace(/\n/g, '\n' + indent),
                    });
                }
            }
        },
    });
    return {
        insertions,
        result: withInsertions(docString, insertions),
    };
}
export function defaultGetDefaultFieldNames(type) {
    if (!('getFields' in type)) {
        return [];
    }
    const fields = type.getFields();
    const leafFieldNames = [];
    Object.keys(fields).forEach(fieldName => {
        if (isLeafType(fields[fieldName].type)) {
            leafFieldNames.push(fieldName);
        }
    });
    return leafFieldNames;
}
export function buildSelectionSet(field, _type, getDefaultFieldNames, schema) {
    var _a;
    const { type } = field;
    if (!type || isLeafType(type)) {
        return;
    }
    const namedType = isNamedType(type)
        ? schema === null || schema === void 0 ? void 0 : schema.getType(type === null || type === void 0 ? void 0 : type.name) : schema === null || schema === void 0 ? void 0 : schema.getType((_a = type === null || type === void 0 ? void 0 : type.ofType) === null || _a === void 0 ? void 0 : _a.name);
    if (!namedType) {
        return;
    }
    const fieldNames = getDefaultFieldNames(namedType);
    if (!Array.isArray(fieldNames) ||
        fieldNames.length === 0 ||
        !('getFields' in namedType)) {
        return;
    }
    return {
        kind: Kind.SELECTION_SET,
        selections: fieldNames.map(fieldName => {
            var _a;
            const fieldDef = namedType === null || namedType === void 0 ? void 0 : namedType.getFields()[fieldName];
            const fieldType = fieldDef ? fieldDef.type : null;
            return {
                kind: Kind.FIELD,
                name: {
                    kind: Kind.NAME,
                    value: fieldName,
                },
                directives: [],
                arguments: (_a = fieldDef === null || fieldDef === void 0 ? void 0 : fieldDef.args) === null || _a === void 0 ? void 0 : _a.map((arg) => {
                    return {
                        name: { kind: 'Name', value: arg.name },
                        kind: 'Argument',
                        value: { kind: 'IntValue', value: '50' },
                    };
                }),
                selectionSet: buildSelectionSet(field, fieldType, getDefaultFieldNames),
            };
        }),
    };
}
function withInsertions(initial, insertions) {
    if (insertions.length === 0) {
        return initial;
    }
    let edited = '';
    let prevIndex = 0;
    insertions.forEach(({ index, string }) => {
        edited += initial.slice(prevIndex, index) + string;
        prevIndex = index;
    });
    edited += initial.slice(prevIndex);
    return edited;
}
function getIndentation(str, index) {
    let indentStart = index;
    let indentEnd = index;
    while (indentStart) {
        const c = str.charCodeAt(indentStart - 1);
        if (c === 10 || c === 13 || c === 0x2028 || c === 0x2029) {
            break;
        }
        indentStart--;
        if (c !== 9 && c !== 11 && c !== 12 && c !== 32 && c !== 160) {
            indentEnd = indentStart;
        }
    }
    return str.substring(indentStart, indentEnd);
}
function isFieldType(fieldType) {
    if (fieldType) {
        return fieldType;
    }
}
//# sourceMappingURL=fillLeafs.js.map