import { GraphQLList, isEnumType, isInputObjectType, isListType, isNonNullType, isScalarType, } from 'graphql';
const scalarTypesMap = {
    Int: 'integer',
    String: 'string',
    Float: 'number',
    ID: 'string',
    Boolean: 'boolean',
    DateTime: 'string',
};
const scalarType = (definition, type) => {
    var _a;
    definition.type = (_a = scalarTypesMap[type.name]) !== null && _a !== void 0 ? _a : 'any';
};
const listType = (definition, type, definitions) => {
    definition.type = 'array';
    const { definition: def, definitions: defs } = getJSONSchemaFromGraphQLType(type.ofType);
    definition.items = { $ref: def.$ref };
    definitions = Object.assign(Object.assign({}, definitions), defs);
};
const enumType = (definition, type) => {
    definition.type = 'string';
    definition.enum = type.getValues().map(val => val.name);
};
const inputObjectType = (definition, type, definitions, options) => {
    definition.$ref = `#/definitions/${type.name}`;
    definition.description = undefined;
    if (!definitions || !definitions[type.name]) {
        const fields = type.getFields();
        const fieldDef = {
            type: 'object',
            properties: {},
            required: [],
        };
        if (type.description) {
            fieldDef.description = type.description;
        }
        if (options === null || options === void 0 ? void 0 : options.useMarkdownDescription) {
            fieldDef.markdownDescription = type.description;
        }
        Object.keys(fields).forEach(fieldName => {
            const field = fields[fieldName];
            const { required, definition: typeDefinition, } = getJSONSchemaFromGraphQLType(field.type, options);
            const { definition: fieldDefinition } = getJSONSchemaFromGraphQLType(fields[fieldName], options);
            fieldDef.properties[fieldName] = Object.assign(Object.assign({}, typeDefinition), fieldDefinition);
            if (required) {
                fieldDef.required.push(fieldName);
            }
        });
        definitions[type.name] = fieldDef;
    }
    else {
        return;
    }
};
export const defaultJSONSchemaOptions = {
    useMarkdownDescription: false,
};
function text(into, newText) {
    into.push(newText);
}
function renderType(into, t) {
    if (!t) {
        return;
    }
    if (isNonNullType(t)) {
        renderType(into, t.ofType);
        text(into, '!');
    }
    else if (t instanceof GraphQLList) {
        text(into, '[');
        renderType(into, t.ofType);
        text(into, ']');
    }
    else {
        text(into, t.name);
    }
}
function renderTypeToString(t, useMarkdown) {
    const into = [];
    if (useMarkdown) {
        text(into, '\n```graphql\n');
    }
    renderType(into, t);
    if (useMarkdown) {
        text(into, '\n```\n');
    }
    return into.join('');
}
function getJSONSchemaFromGraphQLType(type, options) {
    let required = false;
    let definition = {};
    let definitions = {};
    definition.description = renderTypeToString(type);
    const hasDescription = 'description' in type && type.description;
    if ('description' in type && type.description) {
        definition.description += type.description;
    }
    if (options === null || options === void 0 ? void 0 : options.useMarkdownDescription) {
        definition.markdownDescription = renderTypeToString(type, true);
        if (hasDescription) {
            definition.markdownDescription += type.description;
        }
    }
    if ('defaultValue' in type) {
        definition.default = type.defaultValue;
    }
    if (isEnumType(type)) {
        enumType(definition, type);
    }
    if (isInputObjectType(type)) {
        inputObjectType(definition, type, definitions, options);
    }
    if (isScalarType(type)) {
        scalarType(definition, type);
    }
    if (isListType(type)) {
        listType(definition, type, definitions);
    }
    if (isNonNullType(type)) {
        required = true;
        const { definition: def, definitions: defs } = getJSONSchemaFromGraphQLType(type.ofType, options);
        definition = def;
        definitions = Object.assign(Object.assign({}, definitions), defs);
    }
    return { required, definition, definitions };
}
export function getVariablesJSONSchema(variableToType, options) {
    const jsonSchema = {
        $schema: 'https://json-schema.org/draft/2020-12/schema',
        type: 'object',
        properties: {},
        required: [],
    };
    if (variableToType) {
        Object.entries(variableToType).forEach(([variableName, type]) => {
            var _a;
            const { definition, required, definitions, } = getJSONSchemaFromGraphQLType(type, options);
            jsonSchema.properties[variableName] = definition;
            if (required) {
                (_a = jsonSchema.required) === null || _a === void 0 ? void 0 : _a.push(variableName);
            }
            if (definitions) {
                jsonSchema.definitions = definitions;
            }
        });
    }
    return jsonSchema;
}
//# sourceMappingURL=getVariablesJSONSchema.js.map